package com.ruoyi.bst.order.service.impl;

import com.ruoyi.bst.order.domain.OrderBO;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.domain.enums.OrderStatus;
import com.ruoyi.bst.order.service.OrderValidator;
import com.ruoyi.bst.orderProd.domain.OrderProdBO;
import com.ruoyi.bst.orderProd.domain.enums.OrderProdWorkType;
import com.ruoyi.bst.prodProcess.domain.ProdProcessBO;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author wjh
 * 2024/12/17
 */
@Service
public class OrderValidatorImpl implements OrderValidator {


    @Override
    public void checkBefore(OrderBO data) {
        ServiceUtil.assertion(data == null, "订单数据为空");

        // 校验旧订单状态
        if (data.getId() != null) {
            OrderVO old = data.getOld();
            ServiceUtil.assertion(old == null, "待修改的订单数据不存在");
            ServiceUtil.assertion(!OrderStatus.canEdit().contains(old.getStatus()), "订单当前状态不允许修改");

            if (OrderStatus.RELEASED.getStatus().equals(data.getStatus())) {
                ServiceUtil.assertion(!OrderStatus.canRelease().contains(old.getStatus()), "订单当前状态不允许发布");
            }
        }

        // 有且仅有一个结束的产品
        boolean singleEndProd = this.hasSingleEndProd(data.getProdList());
        ServiceUtil.assertion(!singleEndProd, "订单中必须有且仅有一个产品为成品");

        // 每个产品有且仅有一个结束的工序
        for (OrderProdBO prod : data.getProdList()) {
            // 外派无需工序
            if (OrderProdWorkType.OUT.getType().equals(prod.getWorkType())) {
                ServiceUtil.assertion(!CollectionUtils.isEmpty(prod.getProcessList()), "外派产品【%s】无需工序", prod.getName());
            }
            // 自加工
            else {
                boolean singleEndProcess = this.hasSingleEndProcess(prod.getProcessList());
                ServiceUtil.assertion(!singleEndProcess, "产品【%s】校验失败：必须有且仅有一个工序作为产品最终工序", prod.getName());
            }
        }
    }

    private boolean hasSingleEndProcess(List<ProdProcessBO> processList) {
        if (CollectionUtils.isEmptyElement(processList)) {
            return false;
        }

        int endCount = 0;
        for (ProdProcessBO process : processList) {
            if (process.getIsEnd() != null && process.getIsEnd()) {
                endCount ++;
            }
        }

        return endCount == 1;
    }

    private boolean hasSingleEndProd(List<OrderProdBO> prodList) {
        if (CollectionUtils.isEmptyElement(prodList)) {
            return false;
        }

        int endCount = 0;
        for (OrderProdBO prod : prodList) {
            if (prod.getIsEnd() != null && prod.getIsEnd()) {
                endCount ++;
            }
        }

        return endCount == 1;
    }

    @Override
    public void checkAfter(OrderVO vo) {

    }
}
