package com.ruoyi.web.bst;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.bst.logImport.domain.enums.LogImportBizType;
import com.ruoyi.bst.logImportDetail.domain.LogImportDetail;
import com.ruoyi.bst.logImportDetail.domain.LogImportDetailQuery;
import com.ruoyi.bst.logImportDetail.domain.LogImportDetailVO;
import com.ruoyi.bst.logImportDetail.domain.interfaces.ImportContent;
import com.ruoyi.bst.logImportDetail.service.ILogImportDetailService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.LogTitle;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 导入日志明细Controller
 *
 * @author ruoyi
 * @date 2024-11-07
 */
@RestController
@RequestMapping("/bst/logImportDetail")
public class LogImportDetailController extends BaseController
{
    @Autowired
    private ILogImportDetailService logImportDetailService;

    private final static String LOG_TITLE = LogTitle.LOG_IMPORT_DETAIL;

    /**
     * 查询导入日志明细列表
     */
    @PreAuthorize("@ss.hasPermi('bst:logImportDetail:list')")
    @GetMapping("/list")
    public TableDataInfo list(LogImportDetailQuery query)
    {
        startPage();
        startOrderBy();
        List<LogImportDetailVO> list = logImportDetailService.selectLogImportDetailList(query);
        return getDataTable(list);
    }

    /**
     * 导出导入日志明细列表
     */
    @PreAuthorize("@ss.hasPermi('bst:logImportDetail:export')")
    @Log(title = LOG_TITLE, businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, LogImportDetailQuery query) {
        List<LogImportDetailVO> list = logImportDetailService.selectLogImportDetailList(query);
        List<ImportContent> contentList = new ArrayList<>();

        Class<? extends ImportContent> bstClazz = ImportContent.class;  // 业务class类型
        if (CollectionUtils.isNotEmptyElement(list)) {
            // 默认以第一个查到的业务类型作为导出模板
            String bizType = list.stream().map(LogImportDetailVO::getLogBizType).filter(Objects::nonNull).findFirst().orElse(null);
            LogImportBizType logImportBizType = LogImportBizType.parse(bizType);
            if (logImportBizType != null && logImportBizType.getContentClazz() != null) {
                bstClazz = logImportBizType.getContentClazz();
                for (LogImportDetailVO vo : list) {
                    ImportContent content = JSON.parseObject(vo.getContent(), logImportBizType.getContentClazz());
                    content.setImportMessage(vo.getMessage());
                    content.setImportResult(vo.getType());
                    contentList.add(content);
                }
            }
        }

        ExcelUtil util = new ExcelUtil<>(bstClazz);
        util.exportExcel(response, contentList, "导入日志明细数据");
    }

    /**
     * 获取导入日志明细详细信息
     */
    @PreAuthorize("@ss.hasPermi('bst:logImportDetail:query')")
    @GetMapping(value = "/{detailId}")
    public AjaxResult getInfo(@PathVariable("detailId") Long detailId)
    {
        return success(logImportDetailService.selectLogImportDetailByDetailId(detailId));
    }

    /**
     * 新增导入日志明细
     */
    @PreAuthorize("@ss.hasPermi('bst:logImportDetail:add')")
    @Log(title = LOG_TITLE, businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody LogImportDetail logImportDetail)
    {
        return toAjax(logImportDetailService.insertLogImportDetail(logImportDetail));
    }

    /**
     * 修改导入日志明细
     */
    @PreAuthorize("@ss.hasPermi('bst:logImportDetail:edit')")
    @Log(title = LOG_TITLE, businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody LogImportDetail logImportDetail)
    {
        return toAjax(logImportDetailService.updateLogImportDetail(logImportDetail));
    }

    /**
     * 删除导入日志明细
     */
    @PreAuthorize("@ss.hasPermi('bst:logImportDetail:remove')")
    @Log(title = LOG_TITLE, businessType = BusinessType.DELETE)
	@DeleteMapping("/{detailIds}")
    public AjaxResult remove(@PathVariable Long[] detailIds)
    {
        return toAjax(logImportDetailService.deleteLogImportDetailByDetailIds(detailIds));
    }
}
