package com.ruoyi.web.bst;

import com.ruoyi.bst.order.domain.OrderBO;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.service.OrderAssembler;
import com.ruoyi.bst.order.service.OrderConverter;
import com.ruoyi.bst.order.service.OrderService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.validate.ValidGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Collections;
import java.util.List;

/**
 * 生产订单Controller
 *
 * @author ruoyi
 * @date 2024-12-16
 */
@RestController
@RequestMapping("/bst/order")
public class OrderController extends BaseController
{
    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderConverter orderConverter;

    @Autowired
    private OrderAssembler orderAssembler;

    /**
     * 查询生产订单列表
     */
    @PreAuthorize("@ss.hasPermi('bst:order:list')")
    @GetMapping("/list")
    public TableDataInfo list(OrderQuery query)
    {
        startPage();
        startOrderBy();
        List<OrderVO> list = orderService.selectOrderList(query);
        orderAssembler.assembleProgress(list);
        return getDataTable(list);
    }

    /**
     * 导出生产订单列表
     */
    @PreAuthorize("@ss.hasPermi('bst:order:export')")
    @Log(title = "生产订单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, OrderQuery query)
    {
        List<OrderVO> list = orderService.selectOrderList(query);
        ExcelUtil<OrderVO> util = new ExcelUtil<OrderVO>(OrderVO.class);
        util.exportExcel(response, list, "生产订单数据");
    }

    /**
     * 获取生产订单详细信息
     */
    @PreAuthorize("@ss.hasPermi('bst:order:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        OrderVO vo = orderService.selectOrderById(id);
        List<OrderVO> list = Collections.singletonList(vo);
        orderAssembler.assembleProdList(list, true);
        orderAssembler.assembleProgress(list);
        return success(vo);
    }

    /**
     * 新增生产订单
     */
    @PreAuthorize("@ss.hasPermi('bst:order:add')")
    @Log(title = "生产订单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) OrderBO order,
                          @RequestParam(required = false, defaultValue = "false") Boolean submit) {
        OrderBO bo = orderConverter.toBOByCreate(order, submit, getLoginUser().getUser());
        return toAjax(orderService.addOrder(bo));
    }

    /**
     * 修改生产订单
     */
    @PreAuthorize("@ss.hasPermi('bst:order:edit')")
    @Log(title = "生产订单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) OrderBO order,
                           @RequestParam(required = false, defaultValue = "false") Boolean submit)
    {
        OrderBO bo = orderConverter.toBOByUpdate(order, submit, getLoginUser().getUser());
        return toAjax(orderService.editOrder(bo));
    }

    /**
     * 删除生产订单
     */
    @PreAuthorize("@ss.hasPermi('bst:order:remove')")
    @Log(title = "生产订单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{id}")
    public AjaxResult remove(@PathVariable Long id)
    {
        return toAjax(orderService.logicDel(id));
    }
}
